package net.locusworks.test;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileOutputStream;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import net.locusworks.common.interfaces.AutoCloseableIterator;
import net.locusworks.common.utils.FileReader;
import net.locusworks.common.utils.FileReader.LineInfo;
import net.locusworks.common.utils.RandomString;

public class FileReaderTest {

  private static final String TEST_FILE = "test_file.txt";
  private static Map<Integer, Integer> numLines = new LinkedHashMap<>();

  @BeforeClass
  public static void setUpBeforeClass() throws Exception {
    File testFile = new File(TEST_FILE);
    FileOutputStream fos = new FileOutputStream(testFile);

    Integer count = ThreadLocalRandom.current().nextInt(100);

    for (int i = 1; i <= count; i++) {
      String randomString = RandomString.getString(ThreadLocalRandom.current().nextInt(5, 100)) + "\n";
      numLines.put(i, randomString.length());
      fos.write(randomString.getBytes());
    }

    fos.close();
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    File file = new File(TEST_FILE);
    file.delete();
  }

  @Test
  public void testForLoop() {
    Integer lineCount = 0;
    for(LineInfo s : new FileReader(Paths.get(TEST_FILE))) {
      lineCount++;
      Integer lineNumber = s.getLineNumber();
      Integer lineLength = s.getLine().length();
      Integer mapLineLength = numLines.get(lineNumber);
      assertTrue(lineLength == (mapLineLength-1));
    }
    assertTrue(lineCount == numLines.size());
  }

  @Test
  public void testIterator() {
    Integer lineCount = 0;

    try(AutoCloseableIterator<LineInfo> iter = new FileReader(Paths.get(TEST_FILE))) {
      while(iter.hasNext()) {
        lineCount++;
        LineInfo s = iter.next();
        Integer lineNumber = s.getLineNumber();
        Integer lineLength = s.getLine().length();
        Integer mapLineLength = numLines.get(lineNumber);
        assertTrue(lineLength == (mapLineLength-1));
      }
    }
    assertTrue(lineCount == numLines.size());
  }

  @Test
  public void testForIterator() {
    Integer lineCount = 0;
    for(Iterator<LineInfo> iter = new FileReader(Paths.get(TEST_FILE)); iter.hasNext();) {
      lineCount++;
      LineInfo s = iter.next();
      Integer lineNumber = s.getLineNumber();
      Integer lineLength = s.getLine().length();
      Integer mapLineLength = numLines.get(lineNumber);
      assertTrue(lineLength == (mapLineLength-1));
    }
    assertTrue(lineCount == numLines.size());
  }

}
